@@ -76,14 +76,16 @@
 flexdog <- function(refvec,
                     sizevec,
                     ploidy,
-                    model       = c("norm", "hw", "bb", "ash", "s1", "s1pp", "f1", "f1pp", "flex", "uniform"),
+                    model       = c("norm", "hw", "bb", "ash", "s1", "s1pp", "f1", "f1pp", "flex", "uniform", "custom"),
                     p1ref       = NULL,
                     p1size      = NULL,
                     p2ref       = NULL,
                     p2size      = NULL,
+                    snpname     = NULL,
                     bias_init   = exp(c(-1, -0.5, 0, 0.5, 1)),
                     verbose     = TRUE,
                     outliers    = FALSE,
+                    prior_vec   = NULL,
                     ...) {
   assertthat::assert_that(all(bias_init > 0))
   model <- match.arg(model)
@@ -102,17 +104,19 @@ flexdog <- function(refvec,
       cat("Initial Bias:", bias_init[bias_index], "\n")
     }
 
-    fcurrent <- flexdog_full(refvec   = refvec,
-                             sizevec  = sizevec,
-                             ploidy   = ploidy,
-                             model    = model,
-                             p1ref    = p1ref,
-                             p1size   = p1size,
-                             p2ref    = p2ref,
-                             p2size   = p2size,
-                             bias     = bias_init[bias_index],
-                             verbose  = FALSE,
-                             outliers = outliers,
+    fcurrent <- flexdog_full(refvec    = refvec,
+                             sizevec   = sizevec,
+                             ploidy    = ploidy,
+                             model     = model,
+                             p1ref     = p1ref,
+                             p1size    = p1size,
+                             p2ref     = p2ref,
+                             p2size    = p2size,
+                             snpname   = snpname,
+                             bias      = bias_init[bias_index],
+                             verbose   = FALSE,
+                             outliers  = outliers,
+                             prior_vec = prior_vec,
                              ...)
 
     if (verbose) {
@@ -156,7 +160,9 @@ flexdog <- function(refvec,
 #'       default.}
 #'   \item{\code{"hw"}}{A binomial distribution that results from assuming that
 #'       the population is in Hardy-Weinberg equilibrium (HWE). This actually does
-#'       pretty well even when there are minor to moderate deviations from HWE.}
+#'       pretty well even when there are minor to moderate deviations from HWE.
+#'       Though it does not perform as well as the `"norm"` option when there
+#'       are severe deviations from HWE.}
 #'   \item{\code{"bb"}}{A beta-binomial distribution. This is an overdispersed
 #'       version of \code{"hw"} and can be derived from a special case of the Balding-Nichols model.}
 #'   \item{\code{"ash"}}{Any unimodal prior. This can sometimes be sensitive to violations
@@ -190,9 +196,12 @@ flexdog <- function(refvec,
 #'       greater than \code{6}.}
 #'   \item{\code{"flex"}}{Generically any categorical distribution. Theoretically,
 #'       this works well if you have a lot of individuals. In practice, it seems to
-#'       be less robust to violations in modeling assumptions.}
+#'       be much less robust to violations in modeling assumptions.}
 #'   \item{\code{"uniform"}}{A discrete uniform distribution. This should never
 #'       be used in practice.}
+#'   \item{\code{"custom"}}{A pre-specified prior distribution. You specify
+#'       it using the \code{prior_vec} argument. You should almost never
+#'       use this option in practice.}
 #' }
 #'
 #' You might think a good default is \code{model = "uniform"} because it is
@@ -254,6 +263,9 @@ flexdog <- function(refvec,
 #'     error rate.
 #' @param var_seq The prior variance of the logit of the sequencing
 #'     error rate.
+#' @param mean_od The prior mean of the logit of the overdispersion parameter.
+#' @param var_od The prior variance of the logit of the overdispersion
+#'     parameter.
 #' @param seq The starting value of the sequencing error rate.
 #' @param bias The starting value of the bias.
 #' @param od The starting value of the overdispersion parameter.
@@ -295,6 +307,8 @@ flexdog <- function(refvec,
 #'     \code{model = "f1"} (or \code{model = "f1pp"}).
 #' @param p2size The total counts for the second parent if
 #'     \code{model = "f1"} (or \code{model = "f1pp"}).
+#' @param snpname A string. The name of the SNP under consideration.
+#'     This is just returned in the \code{input} list for your reference.
 #' @param ashpen The penalty to put on the unimodal prior.
 #'     Larger values shrink the unimodal prior towards the
 #'     discrete uniform distribution.
@@ -302,6 +316,10 @@ flexdog <- function(refvec,
 #'     (\code{TRUE}) or not (\code{FALSE}). Only supported when
 #'     \code{model = "f1"} or \code{model = "s1"}. I wouldn't
 #'     recommend it for any other model anyway.
+#' @param prior_vec The pre-specified genotype distribution. Only used if
+#'     \code{model = "custom"} and must otherwise be \code{NULL}. If specified,
+#'     then it should be a vector of length \code{ploidy + 1} with
+#'     non-negative elements that sum to 1.
 #'
 #' @return An object of class \code{flexdog}, which consists
 #'     of a list with some or all of the following elements:
@@ -353,6 +371,7 @@ flexdog <- function(refvec,
 #'           \code{get_bivalent_probs(ploidy)$probmat[get_bivalent_probs(ploidy)$lvec == p2geno, , drop = FALSE]}.}
 #'       \item{\code{model = "flex"}:}{\code{par} is an empty list.}
 #'       \item{\code{model = "uniform"}:}{\code{par} is an empty list.}
+#'       \item{\code{model = "custom"}:}{\code{par} is an empty list.}
 #'       }}
 #'   \item{\code{geno}}{The posterior mode genotype. These are your
 #'       genotype estimates.}
@@ -374,6 +393,7 @@ flexdog <- function(refvec,
 #'   \item{\code{input$p1size}}{The value of \code{p1size} provided by the user.}
 #'   \item{\code{input$p2ref}}{The value of \code{p2ref} provided by the user.}
 #'   \item{\code{input$p2size}}{The value of \code{p2size} provided by the user.}
+#'   \item{\code{input$snpname}}{The value of \code{snpname} provided by the user.}
 #'   \item{\code{prop_mis}}{The posterior proportion of individuals
 #'       genotyped incorrectly.}
 #'   \item{\code{out_prop}}{The estimated proportion of points that
@@ -385,11 +405,7 @@ flexdog <- function(refvec,
 #'
 #' @author David Gerard
 #'
-#' @references Gerard, David, Luis Felipe Ventorim Ferrao,
-#' Antonio Augusto Franco Garcia, and Matthew Stephens. 2018.
-#' "Harnessing Empirical Bayes and Mendelian Segregation
-#' for Genotyping Autopolyploids from Messy Sequencing Data."
-#' \emph{bioRxiv}. Cold Spring Harbor Laboratory. doi:10.1101/281550.
+#' @references Gerard, D., Ferrao, L. F. V., Garcia, A. A. F., & Stephens, M. (2018). Genotyping Polyploids from Messy Sequencing Data. *Genetics*, 210(3), 789-807. doi: [10.1534/genetics.118.301468](https://doi.org/10.1534/genetics.118.301468).
 #'
 #' @seealso
 #' Run \code{browseVignettes(package = "updog")} in R for example usage.
@@ -431,12 +447,14 @@ flexdog <- function(refvec,
 flexdog_full <- function(refvec,
                          sizevec,
                          ploidy,
-                         model       = c("norm", "hw", "bb", "ash", "s1", "s1pp", "f1", "f1pp", "flex", "uniform"),
+                         model       = c("norm", "hw", "bb", "ash", "s1", "s1pp", "f1", "f1pp", "flex", "uniform", "custom"),
                          verbose     = TRUE,
                          mean_bias   = 0,
                          var_bias    = 0.7 ^ 2,
                          mean_seq    = -4.7,
                          var_seq     = 1,
+                         mean_od     = -5.5,
+                         var_od      = 0.5 ^ 2,
                          seq         = 0.005,
                          bias        = 1,
                          od          = 0.001,
@@ -453,7 +471,9 @@ flexdog_full <- function(refvec,
                          p1size      = NULL,
                          p2ref       = NULL,
                          p2size      = NULL,
-                         outliers    = FALSE) {
+                         snpname     = NULL,
+                         outliers    = FALSE,
+                         prior_vec   = NULL) {
 
   ## Check input -----------------------------------------------------
   model <- match.arg(model)
@@ -487,7 +507,8 @@ flexdog_full <- function(refvec,
   assertthat::are_equal(1, length(verbose), length(mean_bias),
                         length(var_bias), length(mean_seq),
                         length(var_seq), length(seq),
-                        length(bias), length(od))
+                        length(bias), length(od),
+                        length(mean_od), length(var_od))
   assertthat::assert_that(var_bias > 0)
   assertthat::assert_that(var_seq > 0)
   assertthat::assert_that(seq >= 0, seq <= 1)
@@ -509,6 +530,18 @@ flexdog_full <- function(refvec,
   assertthat::are_equal(length(fs1_alpha), 1)
   assertthat::assert_that(fs1_alpha <= 1, fs1_alpha >= 0)
 
+  ## Check custom --------------------------------------------------
+  if (!is.null(prior_vec) & model != "custom") {
+    stop('Cannot specify `prior_vec` when model != "custom"')
+  } else if (!is.null(prior_vec)) {
+    stopifnot(prior_vec >= 0)
+    stopifnot(prior_vec <= 1)
+    stopifnot(abs(sum(prior_vec) - 1) < 10^-6)
+    stopifnot(length(prior_vec) == ploidy + 1)
+  } else if (model == "custom" & is.null(prior_vec)) {
+    stop('must specify `prior_vec` when model = "custom"')
+  }
+
   ## Check p1ref, p2ref, p1size, p2size ----------------------------
   if ((!is.null(p1ref) | !is.null(p1size)) & (model != "f1" & model != "s1" & model != "f1pp" & model != "s1pp" & model != "f1ppdr" & model != "s1ppdr")) {
     stop("flexdog: if model is not 'f1', 's1', 'f1pp', 's1pp', 'f1ppdr', or 's1ppdr' then p1ref and p1size both need to be NULL.")
@@ -528,14 +561,18 @@ flexdog_full <- function(refvec,
   if (!is.null(p2ref)) {
     stopifnot(p2ref >= 0, p2size >= p2ref)
   }
+  if (!is.null(snpname)) {
+    stopifnot(is.character(snpname))
+    stopifnot(length(snpname) == 1)
+  }
 
   ## check and set mode under various models -----------------------
   if (!is.null(mode) & model == "flex") {
     stop('flexdog: `model` cannot equal `"flex"` when `mode` is specified.')
   } else if (is.null(mode) & model == "flex") {
     mode_vec <- 0
-  } else if (!is.null(mode) & (model == "f1" | model == "s1" | model == "uniform" | model == "f1pp" | model == "s1pp" | model == "f1ppdr" | model == "s1ppdr")) {
-    stop('flexdog: `model` cannot equal "f1", "s1", "f1pp", "s1pp", "f1ppdr", "s1ppdr", or "uniform" when `mode` is specified.')
+  } else if (!is.null(mode) & (model == "f1" | model == "s1" | model == "uniform" | model == "f1pp" | model == "s1pp" | model == "f1ppdr" | model == "s1ppdr" | model == "custom")) {
+    stop('flexdog: `model` cannot equal "f1", "s1", "f1pp", "s1pp", "f1ppdr", "s1ppdr", "uniform", or "custom" when `mode` is specified.')
   } else if (is.null(mode) & (model == "f1" | model == "s1" | model == "uniform" | model == "f1pp" | model == "s1pp" | model == "f1ppdr" | model == "s1ppdr")) {
     mode_vec <- mean(refvec / sizevec, na.rm = TRUE) ## just to initialize pivec
   } else if (!is.null(mode) & model == "ash") {
@@ -549,6 +586,8 @@ flexdog_full <- function(refvec,
     if (any((mode < 0) | (mode > 1))) {
       stop('If model = "hw" or model = "bb" or model = "norm" then `mode` should be between 0 and 1.\nIt is the initialization of the allele frequency.')
     }
+  } else if (model == "custom") {
+    mode_vec <- sum((0:ploidy) * prior_vec) / ploidy
   } else {
     stop("flexdog: Checking mode. How did you get here?")
   }
@@ -649,11 +688,16 @@ flexdog_full <- function(refvec,
         }
       }
       control$mixing_pen <- control$blist$penvec * 1 ## The penalties to use on the mixing weights of the ppdr model.
+    } else if (model == "custom") {
+      control$prior_vec <- prior_vec
     }
 
     ## Initialize pivec so that two modes have equal prob if model = "ash".
     ##     Uniform if model = "flex".
     pivec <- initialize_pivec(ploidy = ploidy, mode = mode, model = model)
+    if (model == "custom") { ## hack to get around passing more arguments to initialize_pivec
+      pivec <- prior_vec
+    }
     assertthat::are_equal(sum(pivec), 1)
     control$pivec <- pivec ## initialization for unimodal optimization
 
@@ -705,6 +749,8 @@ flexdog_full <- function(refvec,
                            var_bias    = var_bias,
                            mean_seq    = mean_seq,
                            var_seq     = var_seq,
+                           mean_od     = mean_od,
+                           var_od      = var_od,
                            wmat        = wik_mat,
                            update_seq  = update_seq,
                            update_bias = update_bias,
@@ -792,7 +838,9 @@ flexdog_full <- function(refvec,
                              mean_bias = mean_bias,
                              var_bias  = var_bias,
                              mean_seq  = mean_seq,
-                             var_seq   = var_seq)
+                             var_seq   = var_seq,
+                             mean_od   = mean_od,
+                             var_od    = var_od)
       } else {
         llike <- flexdog_obj_out(probk_vec = probk_vec,
                                  out_prop  = out_prop,
@@ -805,7 +853,9 @@ flexdog_full <- function(refvec,
                                  mean_bias = mean_bias,
                                  var_bias  = var_bias,
                                  mean_seq  = mean_seq,
-                                 var_seq   = var_seq)
+                                 var_seq   = var_seq,
+                                 mean_od   = mean_od,
+                                 var_od    = var_od)
       }
 
 
@@ -890,6 +940,7 @@ flexdog_full <- function(refvec,
   return_list$input$p1size  <- p1size
   return_list$input$p2ref   <- p2ref
   return_list$input$p2size  <- p2size
+  return_list$input$snpname <- snpname
   return_list$prop_mis      <- 1 - mean(return_list$maxpostprob)
 
   ## Add back missingness ---------------------------------------------
@@ -1012,7 +1063,7 @@ initialize_pivec <- function(ploidy,
                              model = c("hw", "bb", "norm", "ash",
                                        "f1", "s1", "f1pp", "s1pp",
                                        "f1ppdr", "s1ppdr",
-                                       "flex", "uniform")) {
+                                       "flex", "uniform", "custom")) {
   assertthat::are_equal(1, length(ploidy), length(mode))
   assertthat::are_equal(ploidy %% 1, 0)
 
@@ -1050,7 +1101,7 @@ initialize_pivec <- function(ploidy,
     } else if (init_type == "piunif") {
       pivec <- rep(1 / (ploidy + 1), length = ploidy + 1)
     }
-  } else if (model == "hw" | model == "f1" | model == "s1" | model == "bb" | model == "norm" | model == "f1pp" | model == "s1pp" | model == "f1ppdr" | model == "s1ppdr") {
+  } else if (model == "hw" | model == "f1" | model == "s1" | model == "bb" | model == "norm" | model == "f1pp" | model == "s1pp" | model == "f1ppdr" | model == "s1ppdr" | model == "custom") {
     if (mode < 0 | mode > 1) {
       stop('initialize_pivec: when model = "hw", mode should be between 0 and 1.\n It is the initialization of the allele frequency.')
     }
@@ -1085,7 +1136,7 @@ flex_update_pivec <- function(weight_vec,
                                         "ash", "f1", "s1",
                                         "f1pp", "s1pp",
                                         "f1ppdr", "s1ppdr",
-                                        "flex", "uniform"),
+                                        "flex", "uniform", "custom"),
                               control) {
   ## Check input -------------------------------
   ploidy <- length(weight_vec) - 1
@@ -1259,6 +1310,9 @@ flex_update_pivec <- function(weight_vec,
       return_list$par$p2_pair_weights  <- temp_list$p2_pair_weights[[temp_list$p2geno + 1]]
       return_list$par$p2_dr_proportion <- sum(temp_list$p2_pair_weights[[temp_list$p2geno + 1]][!control$blist$penvec[control$blist$lvec == temp_list$p2geno]])
     }
+  } else if (model == "custom") {
+    return_list$pivec <- control$prior_vec
+    return_list$par <- list()
   } else {
     stop("flex_update_pivec: how did you get here?")
   }
